using System.Collections.Generic;
using System.Linq;
using MacGruber;
using MVR.FileManagementSecure;
using SimpleJSON;
using UnityEngine.UI;

namespace CheesyFX
{
    public class DialogPool
    {
        public static List<List<Dialog>> dialogs = new List<List<Dialog>>();
        private static Dialog last;
        private static string help = "Each pose and cam that has a 'Dialog Pool Level' greater than -1 will play a random dialog out of the level you set.";
        public static bool uiOpen;
        private static int currentLevel;
        private static UIDynamicTabBar tabbar;
        private static JSONStorableUrl loadURL;
        private static List<string> levelNames = new List<string>{ "Lvl 0", "Lvl 1", "Lvl 2" };
        private static string saveDir = "Saves/PluginData/CheesyFX/BodyLanguage/PoseMe/DialogPools/";
        private static JSONNode levelCache;

        public static void Init()
        {
            dialogs.Add(new List<Dialog>());
            dialogs.Add(new List<Dialog>());
            dialogs.Add(new List<Dialog>());
            FileManagerSecure.CreateDirectory(saveDir);
            loadURL = new JSONStorableUrl("loadURL", "", UILoadJSON, "json", true);
            loadURL.hideExtension = true;
            loadURL.allowFullComputerBrowse = false;
            loadURL.allowBrowseAboveSuggestedPath = true;
            loadURL.SetFilePath(saveDir);
            
        }
        
        public static void UILoadJSON(string url)
        {
            if(!FileManagerSecure.FileExists(url)) return;
            Load(SuperController.singleton.LoadJSON(url).AsObject);
            CreateUI();
        }

        private static void AddLevel(List<Dialog> newDialogs = null)
        {
            levelNames.Add($"Lvl {levelNames.Count}");
            if(newDialogs == null) dialogs.Add(new List<Dialog>());
            else dialogs.Add(new List<Dialog>(newDialogs));
            PoseMe.dialogPoolLevelPose.max = levelNames.Count - 1;
            PoseMe.dialogPoolLevelCam.max = levelNames.Count - 1;
            if(uiOpen) CreateUI();
        }

        public static void AddDialog()
        {
            var dialog = new Dialog(isPoolDialog:true);
            float rightItems = dialogs[currentLevel].Count(x => x.uidItem.rightSide);
            var rightSide = rightItems < dialogs[currentLevel].Count / 2f;
            dialogs[tabbar.id].Add(dialog);
            
            PoseMe.UIElements.Add(PoseMe.CreateDialogUIItem(dialog, rightSide));
        }
        
        public static void AddDialog(JSONClass jc)
        {
            var dialog = new Dialog(jc)
            {
                isPoolDialog = true
            };
            float rightItems = dialogs[currentLevel].Count(x => x.uidItem.rightSide);
            var rightSide = rightItems < dialogs[currentLevel].Count / 2f;
            dialogs[tabbar.id].Add(dialog);
            
            PoseMe.UIElements.Add(PoseMe.CreateDialogUIItem(dialog, rightSide));
        }
        
        public static void AddDialog(JSONClass jc, int level)
        {
            var dialog = new Dialog(jc);
            dialog.isPoolDialog = true;
            dialogs[level].Add(dialog);
        }

        public static void RemoveDialog(Dialog dialog)
        {
            dialogs[tabbar.id].Remove(dialog);
            PoseMe.singleton.RemoveUIElement(dialog.uidItem);
            last = null;
        }
        
        public static void InvokeRandom(int level)
        {
            if (level < 0) return;
            last = dialogs[level].TakeRandom(last);
            last.Invoke();
        }

        public static void CreateUI()
        {
            PoseMe.singleton.ClearUI();
            var button = PoseMe.singleton.CreateButton("Return");
            button.buttonColor = PoseMe.navColor;
            button.button.onClick.AddListener(CloseUI);
            
            // Utils.SetupTwinButton(PoseMe.singleton, "Add Dialog", AddDialog, "Add Level", () => AddLevel(), false);
            
            
            PoseMe.singleton.SetupButton("Help", true, () => help.Print());
            PoseMe.singleton.SetupButton("Copy Level", true, CopyLevel);
            var twinButton = Utils.SetupTwinButton(PoseMe.singleton, "Save Preset", UISaveJSONDialog, "Merge Load", delegate {}, false);
            loadURL.RegisterFileBrowseButton(twinButton.buttonRight);
            
            Utils.SetupTwinButton(PoseMe.singleton, "Add Dialog", AddDialog,"Paste Dialog", () => AddDialog(Dialog.cache), false);
            Utils.SetupTwinButton(PoseMe.singleton, "Add Level", () => AddLevel(),"Merge Paste Level", PasteLevel, false);
            
            twinButton = Utils.SetupTwinButton(PoseMe.singleton, "Move Level Up", () => MoveLevel(true), "Move Level Down", () => MoveLevel(false), true);
            
            twinButton = Utils.SetupTwinButton(PoseMe.singleton, "Clear Level", ClearLevel,"Delete Last Level", DeleteLastLevel, true);
            twinButton.buttonLeft.GetComponentInChildren<Image>().color = PoseMe.severeWarningColor;
            twinButton.buttonRight.GetComponentInChildren<Image>().color = PoseMe.severeWarningColor;
            // PoseMe.singleton.SetupButton("Clear Level", ClearLevel, PoseMe.severeWarningColor);
            // PoseMe.singleton.SetupButton("Delete Last Level", DeleteLastLevel, PoseMe.severeWarningColor, rightSide:true);
            tabbar = UIManager.CreateTabBar(levelNames.ToArray(), UISelectLevel);
            tabbar.SelectTab(currentLevel);

            uiOpen = true;
        }

        private static void CopyLevel()
        {
            levelCache = StoreLevel(currentLevel);
        }

        private static void PasteLevel()
        {
            if (levelCache == null)
            {
                "Copy a level first.".Print();
                return;
            }
            MergeLoadLevel(currentLevel, levelCache);
            tabbar.SelectTab(currentLevel);
        }

        private static void DeleteLastLevel()
        {
            var id = levelNames.Count - 1;
            if(id == currentLevel) currentLevel--;
            levelNames.RemoveAt(id);
            dialogs.RemoveAt(id);
            PoseMe.dialogPoolLevelPose.max = levelNames.Count - 1;
            PoseMe.dialogPoolLevelCam.max = levelNames.Count - 1;
            if(uiOpen) CreateUI();
        }
        
        private static void ClearLevel()
        {
            foreach (var bubble in dialogs[currentLevel])
            {
                PoseMe.singleton.RemoveUIElement(bubble.uidItem);
            }
            dialogs[currentLevel].Clear();
            tabbar.SelectTab(currentLevel);
        }

        private static void MoveLevel(bool up)
        {
            if (up)
            {
                if (currentLevel == dialogs.Count-1)
                {
                    AddLevel(dialogs[currentLevel]);
                    dialogs[currentLevel].Clear();
                }
                else
                {
                    var nextLevel = dialogs[currentLevel + 1];
                    dialogs[currentLevel + 1] = dialogs[currentLevel];
                    dialogs[currentLevel] = nextLevel;
                }
                currentLevel++;
            }
            else
            {
                if (currentLevel == 0) return;
                var previousLevel = dialogs[currentLevel - 1];
                dialogs[currentLevel - 1] = dialogs[currentLevel];
                dialogs[currentLevel] = previousLevel;
                currentLevel--;
            }
            tabbar.SelectTab(currentLevel);
        }
        
        private static void UISaveJSONDialog()
        {
            SuperController.singleton.GetMediaPathDialog(UISaveJSON, "json", saveDir, false, true, false, null, true, null, false, false);
            SuperController.singleton.mediaFileBrowserUI.SetTextEntry(true);
            if(SuperController.singleton.mediaFileBrowserUI.fileEntryField != null){
                string filename = PoseMe.sceneName;
                SuperController.singleton.mediaFileBrowserUI.fileEntryField.text = filename;
                SuperController.singleton.mediaFileBrowserUI.ActivateFileNameField();
            }
        }
        
        private static void UISaveJSON(string path)
        {
            if (string.IsNullOrEmpty(path)) return;
            string path1 = path.Replace('\\', '/')+".json";
            JSONClass jc = Store();
            SuperController.singleton.SaveJSON(jc, path1);
            SuperController.singleton.DoSaveScreenshot(path+".jpg");
        }

        private static void UISelectLevel(int id)
        {
            foreach (var bubble in dialogs[currentLevel])
            {
                PoseMe.singleton.RemoveUIElement(bubble.uidItem);
            }
            CreateItems(id);
            currentLevel = id;
        }

        private static void CreateItems(int level)
        {
            for (int i = 0; i < dialogs[level].Count; i++)
            {
                PoseMe.UIElements.Add(PoseMe.CreateDialogUIItem(dialogs[level][i], i%2 == 1));
            }
        }

        private static void CloseUI()
        {
            PoseMe.singleton.ClearUI();
            uiOpen = false;
            PoseMe.singleton.CreateUI();
        }

        public static JSONClass Store()
        {
            var jc = new JSONClass();

            for (int i = 0; i < dialogs.Count; i++)
            {
                jc[$"Lvl{i}"] = StoreLevel(i);
            }

            return jc;
        }

        private static JSONArray StoreLevel(int i)
        {
            var ja = new JSONArray();
            foreach (var dialog in dialogs[i])
            {
                ja.Add(dialog.Store());
            }
            return ja;
        }

        public static void Load(JSONClass jc)
        {
            if (!jc.HasKey("Lvl0")) return;
            var levels = jc.Childs.ToArray();
            for (int i = 0; i < levels.Length; i++)
            {
                MergeLoadLevel(i, levels[i]);
            }
        }

        private static void MergeLoadLevel(int i, JSONNode ja)
        {
            if(i == dialogs.Count) AddLevel();
            foreach (var bubble in ja.Childs)
            {
                AddDialog(bubble.AsObject, i);
            }
        }
    }
}